#!/usr/bin/perl -w -I/usr/local/share/perl/5.8.0:/usr/share/plm/lib

use strict;

use BASE::Log;
use BASE::Config;
use PLMClient;
use PLM;

my $log = getLog();
my $cfg = getConfig();

my ( $repo, $patch_id ) = @ARGV;

unless ( $repo )     { panic( "Missing software repository" ) }
unless ( $patch_id ) { panic( "Missing patch ID number to build tree of" ) }

my @applies;

sanity_check();

build_tree( $_ );

sub sanity_check {
    if ( -d $repo ) {
        print "ERROR - the $repo directory already exists\n";
        exit 1;
    }
}

sub build_tree {
    @applies = ASP( "get_applies_tree", $patch_id );
    unless ( scalar @applies ) {
        print "[$patch_id] is an invalid patch ID.\n";
        exit 1;
    }
    print "Applies Tree: @applies\n";

    my $base_patch = shift @applies;
    handle_base( $base_patch );
    for ( @applies ) {
        handle_patch( $_ );
    }
}

sub handle_base {
    my $pid = shift;

    return if ( -d $repo );    # Another part of tree already expanded to base

    my ( $filename, $base_location ) = ASP( "get_base_filename", $pid );
    panic( "Can't get filename or location from ASP server." )
      unless $filename;

    my $location = $cfg->get( "repo_$repo" . "_location" );
    panic( "Can't get repo location from config" ) unless $location;

    my $locType = $cfg->get( "repo_$repo" . "_type" );
    panic( "Can't get repo type from config" ) unless $locType;

    my $path = "$location/$base_location/$filename";

    print "[$locType] $path\n";

    if ( $locType =~ /URL/ && !( -f $filename ) ) {
        my $retry = 100;
        while ( $retry && !( -f $filename ) ) {
            print "wget -nv -t50 --waitretry=50 $path\n";
            system "wget -nv -t50 --waitretry=50 $path";
            if ( !( -f $filename ) ) {
                $retry--;
                sleep 10;
            }
        }
    } elsif ( !( -f $filename ) ) {
        panic( "bad location type" );
    }

    if ( !( -f $filename ) ) {
        panic( "download of kernel failed!" );
    }

    print "Unpacking source: $filename\n";
    if ( $filename =~ /bz2$/ ) {
        system "tar -jxf $filename";
    }

    if ( $filename =~ /gz$/ ) {
        system "tar -zxf $filename";
    }

    $filename =~ s/(.*)\.tar.*/$1/;
    system "mv $filename $repo" if ( -d $filename );
}

sub handle_patch {
    my $pid  = shift;
    my $file = "plm-$pid.patch";
    my $path;

    $path = $cfg->get( "getpatch_url" ) || "";
    $path = ( $cfg->get( "plm_http" ) . "/getpatch" ) unless ( $path );

    panic( "Can't get getpatch path from config" ) unless $path;

    unless ( -f "getpatch?id=$pid" || -d $file ) {
        print "wget -nv -t50 --waitretry=50 $path?id=$pid\n";
        system "wget -nv -t50 --waitretry=50 $path?id=$pid";
        panic( "unable to download patch $pid" )
          unless ( -f "getpatch?id=$pid" );
        system "mv getpatch?id=$pid $file";
    }

    chdir $repo;

    print "Patching source, placing output in: patch.$pid.out\n";
    system "(patch -t -p1 < ../$file &> patch.$pid.out) || touch patch.error";

    chdir "..";
}

