#!/usr/bin/perl -w

# a script to test if all the desired devices exist and
# are properly size
# size will be determined by the file /root/disk.info


use strict;

my $DEBUG = 0; 
my $diskfile = "/proc/partitions";
my $disktext = "/root/disk.info";
my $hostname = `hostname`;
die "ERROR: failed to get hostname\n" if ($? != 0);
my $modulefile = "/root/modules";
my $systype = "";
if ( $hostname =~ /^stp/ ) {
	$systype = ($hostname =~ /^stp(\d+)-/)[0]; # leave only the number
	die "ERROR: hostname does not look like an stp machine\n"
		if (!defined($systype));
	$systype+=0; # make sure this number is numeric, not stringed (002 becomes 2)
} else {
	# Skip these tests for now
	exit 0;
}
my $sysstate = 0;


# this is a hardcoded array of expected memory sizes
# Right now the 8-ways boot with a stupid install kernel
my %memsize = (
	1 => 512000,
	2 => 902000,
	4 => 3930000,
	8 => 3800000,
);

my $meminfo = "/proc/meminfo";

# check available CPU's, use hostname to guide us. 
# WARNING - this method may fail with other hostnames

chomp(my $cpu_count = `getconf _NPROCESSORS_ONLN`);
die "ERROR: failed to get processor count\n" if ($? != 0);

unless ( $systype eq $cpu_count ) {
	print "ERROR: only $cpu_count CPU's found, expecting $systype\n";
	$sysstate++;
}

open( F, "<$meminfo" ) || die "Can't open $meminfo $!";
while( my $ml = <F> ) {
	chomp($ml);
	unless ( $ml =~ /^MemTotal/ ) { next; }
	my ( $name, $size, $kb ) = split /\s+/,$ml;
	if ( $DEBUG ) { print "Mem size is $size\n"; }
	unless ( $size >= $memsize{$systype} ) {
		print "ERROR: Memory bad: expected $memsize{$systype} got $size\n";
		$sysstate++;
	}

}
close F;

if ( $systype < 4 ) { 	# no disks to check
	exit $sysstate;
}
# First, we make sure the modules are loaded
`depmod -a`;
open( F, "<$modulefile" ) || die "Can't open /root/modules $!";
while(<F>) {
	chomp;
	my $mod = $_;
	`modprobe $mod`;
	`sleep 30`;
}
close F;

open( F, "<$diskfile" ) || die "Can't open /proc/partitions $!"; 

my %spindles;
my %disks;

# Check /proc/partitions for actual disks
while(<F>) {
	chomp;
	my ( $dummy,  $major, $minor, $blocks, $name ) = split /\s+/;
	unless ( $name ) { next; }
	if ( $major eq "major" ) { next; } 
	if ( $name =~ m/\d+/ ) { next; }
	if ( $DEBUG ) { print "Disk $name size $blocks\n"; } 
	$spindles{ $name } = $blocks; 
}

close F;

open( F, "<$disktext" ) || die "Can't open $disktext $!";

# read the disk.info file
while(my $dl = <F>) {
	chomp($dl);
	if (( $dl =~ /^#/ ) || ( $dl =~ /^$/)) { next; }
	my ( $dev, $fakemnt, $size, $raid, $spindle, $nulls ) = split /,/,$dl;
	$dev =~ s/\/dev\///; 
	$size = $size * 1024;
	if ( $DEBUG ) { print "Found $dev size $size\n"; }
	$disks{ $dev } = $size;
}
close F;

# compare
for my $disk ( keys(%disks) ) {
	unless ( $spindles{ $disk } ) { 
		print "ERROR: Missing $disk \n";
		$sysstate++;
	}
	my $file_size = $disks{ $disk }; 
	my $real_size = $spindles{ $disk };

	unless ( $real_size >= $file_size ) {
		print "ERROR: Bad size $disk $real_size expected $file_size actual\n";
		$sysstate++;
	}
}
	# if we get here, all is happy

print "Disks are okay!\n";
exit $sysstate;
	
	



