#!/usr/bin/perl -w -I/usr/lib/perl5/site_perl/
#

# A wrapper for the STP client tools
use STP::Client::DiskHandler;

use strict;
use English;
use Getopt::Long;
use Pod::Usage;

=head1 NAME

mount_disk.pl

=head1 SYNOPSIS

given a minium size, a filesystem type and a request mount point,
checks state of tools and installs if needed, finds the largest
available unmounted disk, fdisks, makes the filesystem and mounts
Currently supports 

 -ext2
 -ext3
 -reiserfs (v3)
 -xfs
 -jfs
 -raw

=head1 ARGUMENTS

REQUIRED
 -type <File system type>
 -at <name of mount point>, will be created if non-existant
OPTIONAL
 -size <minimum> ( defaults to largest available ) 
 -mkfs <string for over-ridding mkfs, must include 'mk*' >
 -mount <string for mount options, 'mount' assumed>
OVER-RIDE
 -file <filename with mountinfo information>

=cut

my ( $fstype, $minsize, $mnt_point, $mkfs_opt, $mnt_opts, $file, $hlp, $res );

GetOptions(
            "type=s"  => \$fstype,
            "size=i"  => \$minsize,
            "at=s"    => \$mnt_point,
            "mkfs=s"  => \$mkfs_opt,
            "mount=s" => \$mnt_opts,
            "file=s"  => \$file,
            "help"    => \$hlp
);

if ( $hlp ) { pod2usage( 1 ); }

# Our over-ride
if ( $file ) {
    if ( -f $file ) {
        $res = create_disk_layout( $file );
        return $res;
    } else {
        print STDERR "File $file not found!\n";
        return -1;
    }
}

# Otherwise, we are going with the command options
#
unless ( ( $fstype ) && ( $mnt_point ) ) {
    print STDERR "Missing options!\n";
    pod2usage( 1 );
    return -1;
}

# Default size to a minimum if not supplied

unless ( $minsize )  { $minsize  = 1; }
unless ( $mkfs_opt ) { $mkfs_opt = 'NULL'; }
unless ( $mnt_opts ) { $mnt_opts = 'NULL'; }

$res = make_one_disk( $fstype, $minsize, $mnt_point, $mkfs_opt, $mnt_opts );

exit $res;

