#
# Helper routines for STP client sh scripts
#
# This file is part of the client-SDK for the STP
#
# This file MUST be included at the top of the wrap.sh script if it is used
# Include syntax: . stp_include.sh
#

#
# The version of the client API this include file supports
#
export STP_CLIENT_API="2.1.0"

#
# The home directory for the wrap.sh script
#
export WRAP_HOME=`pwd`

#
# The number of CPUs present in the system
#
CPU_COUNT=1
[ -f '/proc/cpuinfo' ] && CPU_COUNT=`grep -c "^processor" /proc/cpuinfo`
export CPU_COUNT

#
# The directory to store the result data in
#
export RESULTS="${WRAP_HOME}/results"
mkdir $RESULTS

#
# The file to create that will be used as the content of the email to the user
# who requested the test.
#
export RESULTS_EMAIL="${WRAP_HOME}/result_email.txt"

#
# The filename of the disk.info file
#
export DISK_INFO="${WRAP_HOME}/disk.info"

#
# Flag indicating some type of profile data will be auto-gathered
#
PROFILE_RUN=0
[ -f '/proc/profile' ] && PROFILE_RUN=1
export PROFILE_RUN

# 
# Distro support - 
# Current Distros include:
# Suse ( /etc/SuSE-release )
# Redhat ( /etc/redhat-release ) 
# Redhat versions are:
# Shrike ( 9_0 )
# Valahalla ( 7_3 ) 
#
DISTRO="unknown"
if [ -f '/etc/SuSE-release' ]; then 
	DISTRO='SuSE'
fi
if [ -f '/etc/redhat-release' ]; then
	VER=`cat /etc/redhat-release | awk '{print $5}'`
	DISTRO="RH_$VER"
fi
export DISTRO

#
# Internal test state management
#
INTERNAL_STATE="unset"
[ -f '/tmp/stp-int-state' ] && INTERNAL_STATE=`cat /tmp/stp-int-state`
export INTERNAL_STATE

set_internal_state() {
    export INTERNAL_STATE="$*"
    echo ${INTERNAL_STATE} > /tmp/stp-int-state
    logger -t STP "Set the internal test state to: [${INTERNAL_STATE}]"
}

#
# Reboot the machine on purpose.  Return to wrap.sh on bootup
# File is one directory up from the $WRAP_HOME
#
request_reboot() {
    echo "CORRECT" > ${WRAP_HOME}/../i-meant-to-reboot
    # Make a seperate log for each segment
    COUNT=0
    while [ -e "run-log.${COUNT}.txt" ] ; do
         COUNT=$((COUNT + 1))
    done
    cp $TEST_LOGS/run-log.txt $TEST_LOGS/run-log.${COUNT}.txt
    logger -t STP Test requested reboot
    reboot
    sleep 5m
}

#
# Load the host-specific modules (not all tests have special HW needs)
#
load_host_modules() {
    logger -t STP Loading host specific hardware modules
    export PATH="/usr/local/sbin:$PATH"
    depmod -a
    if [ -f "${WRAP_HOME}/../modules" ]; then
        for x in `cat ${WRAP_HOME}/../modules`; do
            logger -t STP Loading module "$x"
            modprobe $x || exit 1
            sleep 30
        done
    fi
}

#
# Mount all kernel-specfic psuedo filesystems
# Start with sysfs
# Will be mounted as /sys and /sysfs to be
# sure all the tools work
#
# TODO

#
# Generate a index.html and 
unknown_panic() {
    logger -r STP An unknown panic has been triggered
    exit 1
}



#
# Let the syslog know we finished loading the common routines
#
logger -t STP Common routines loaded OK

#
# Parse the framework request-details file
#
TEST_ID=0
TEST_NAME="unknown"
PATCH_ID=0
PATCH_NAME="unknown"
DETAILS=${WRAP_HOME}/../request-details
if [ -f $DETAILS ]; then
    TEST_ID=`grep Request $DETAILS | cut -c15-`
    TEST_NAME=`grep Test $DETAILS | cut -c15-`
    DISTRO_NAME=`grep "Distro Name" $DETAILS | cut -c15-`
    PATCH_ID=`grep "Kernel ID" $DETAILS | cut -c15-`
    NODE_NAME=`grep "Host Name" $DETAILS | cut -c15-`
    LILO_OPTS=`grep "LILO opt:" $DETAILS | cut -c15-`
    SCRIPT_PARAMS=`grep "Script Params" $DETAILS | cut -c15-`
    PATCH_NAME=`grep "Kernel Name" $DETAILS | cut -c15-`
fi
export TEST_ID TEST_NAME DISTRO_NAME PATCH_ID PATCH_NAME LILO_OPTS SCRIPT_PARAMS

#
# Create the diretory & target for placing log files
#
export TEST_LOGS="${WRAP_HOME}/logs"
mkdir $TEST_LOGS

#
# Handle the HTML result template
#
export RESULTS_HEADER="/tmp/STP-result_header.html"
export RESULTS_FOOTER="/tmp/STP-result_footer.html"

cat << EOF > $RESULTS_HEADER
<html><head>
<title>Scalable Test Platform Test Results</title>
</head>
<body>
<h1>Results for $TEST_NAME</h1>
<h2>Request ID: $TEST_ID ran on $PATCH_NAME [PLM: $PATCH_ID] On Node: $NODE_NAME</h2>
<h3>Distro: $DISTRO_NAME </h3>
<h3>Kernel append: $LILO_OPTS</h3>
<h3>Script Params: $SCRIPT_PARAMS</h3>
<ul>
<li><a href="environment/machine_info">Machine Configuration Summary</a></li>
<li><a href="profile/">Profile Report ( if available )</a></li>
<li><a href="oprofile/">OProfile Report ( if available )</a></li>
</ul>
<p/>
EOF

cat << EOF > $RESULTS_FOOTER
<h2>Informative Links:</h2>
<ul>
<li><a href="COPYING">Results License</a></li>
<li><a href="environment/">Environment Documentation</a></li>
<li><a href="result_email.txt">Summary Email</a> (sent to original requestor)</li>
<li><a href="logs/">Log Outputs</a></li>
</ul>
<br/>
<p/>
</body>
</html>
EOF


#
# Generate a failure email and exit
mesg_and_die() {
 echo $1
        echo $1 >> $RESULTS_EMAIL
        cat $RESULTS_HEADER  >> $WRAP_HOME/index.html
        echo "<PRE>$1</PRE>" >> $WRAP_HOME/index.html
        cat $RESULTS_FOOTER  >> $WRAP_HOME/index.html
        exit $2
}

#
# send timestamp and msg to console

console_note() {
                TIME=`date +'Time: %T %D'` 
        echo $TIME $1 >> /dev/console
}

