
# set -x

export TARG="$WRAP_HOME/oprofile"
export LOGFILE="$WRAP_HOME/oprofile.txt"
export OP_INDEX="$TARG/index.html"
export OP_CNTRL="/usr/local/bin/opcontrol"
export OP_RPT="/usr/local/bin/opreport"
export OP_NOTE="/usr/local/bin/opannotate"
export KERN="/boot/vmlinux"



#
# If this is not the first time we have run on this machine, change our $HOME
# to reflect the original location
#
# if ( -f "/tmp/stp-oprofile-home" ) {
    # $HOME = `cat /tmp/stp-oprofile-home`;
    # chomp $HOME;
    # $TARG = "$HOME/oprofile";
# } else {
    # open( F, ">/tmp/stp-oprofile-home" ) || die "Cannot create home file";
    # print F $HOME;
    # close( F );
# }
#
# Check for profile support and fail with a message if it's missing.
#
check_oprofile() {

    if [ ! -d "/dev/oprofile" ]; then
	
        	console_note "This system is not configured for profile gathering."
        	console_note "To enable profile gathering make sure the kernel has profiling"
        	console_note "enabled and/or the oprofile module is loaded."

        	safe_exit
    fi
}

# Begin the profile gathering process, create the working dir
#
start_oprofile() {
    check_oprofile

    console_note "Starting the oprofile gathering phase"

    echo  "<html><head><title>OProfile Data Report</title></head>" >> $OP_INDEX
    echo  "<body>" >> $OP_INDEX
    echo  "<h1>OProfile Data Report</h1>" >> $OP_INDEX
    echo "OProfile data is available for the following timestamp events.<p>" >> $OP_INDEX
    echo  "<table cellpadding=2 width=700>" >> $OP_INDEX

    kick_daemon
    sleep 1
    mark_oprofile "Framework Startup" 

}

# uses the --save-session option of opcontrol
# resets the counter to zero

mark_oprofile() {
	  check_oprofile

    txtd=`/bin/date +"Event: %T %D"`
    txt="$txtd $1"
    tim=`/bin/date +"%T %D"`
    console_note $txt
    tmps=`echo $1 | sed 's/ /_/g'`
    console_note "Marking oprofile event: $1"
    $OP_CNTRL --dump
    sleep 1
    $OP_CNTRL --save=${tmps}
    sleep 1
    $OP_CNTRL --reset
}
add_to_html() {

    tim=`/bin/date +"%T %D"`
    echo  "<tr>" >> $OP_INDEX
    echo  "<td>$1</td>" >> $OP_INDEX
    echo  "<td>$tim</td>" >> $OP_INDEX
    echo  "<td><a href='$2'>$3</a></td>" >> $OP_INDEX
    echo "</tr>" >> $OP_INDEX

}
report_kern_oprofile() {
   	
    tmps=`echo $1 | sed 's/ /_/g'`
    console_note "Reporting oprofile event: $1"
    $OP_CNTRL --dump
    $OP_RPT -l $KERN -o $TARG/${tmps}-kernel-oprofile.txt session:$tmps
    add_to_html $tmps "${tmps}-kernel-oprofile.txt" "OProfile Kernel data"
}

report_all_oprofile() {
    tmps=`echo $1 | sed 's/ /_/g'`
    console_note "Reporting oprofile event: $1"
    $OP_CNTRL --dump
    $OP_RPT -o $TARG/${tmps}-all-oprofile.txt session:$tmps
    add_to_html $tmps "${tmps}-all-oprofile.txt" "OProfile System data"
}

anote_src_oprofile() {
    # Note: requires debugging info 
    tmps=`echo $1 | sed 's/ /_/g'`
    console_note "Anonnotate oprofile event: $1"
    $OP_CNTRL --stop
    $OP_NOTE --source --output-dir=$TARG/${tmps}-oprofile-annotated 
    kick_daemon
    add_to_html $tmps ${tmps}-oprofile-annotated "OProfile Annotated Source Directory"
}

anote_ass_oprofile() {
    tmps=`echo $1 | sed 's/ /_/g'`
    console_note "Anonnotate oprofile event: $1"
    $OP_CNTRL --stop
    $OP_NOTE --source --assembly session:$tmps > $TARG/${tmps}-oprofile-assembly.txt  2>&1
    kick_daemon
    add_to_html $tmps ${tmps}-oprofile-assembly.txt "OProfile Annotated Assembly"
}
kick_daemon() {

    # Restart life
    console_note "Refreshing oprofile daemon"
    $OP_CNTRL --vmlinux=/root/linux/vmlinux
    sleep 1
    $OP_CNTRL --start-daemon
    sleep 1
    $OP_CNTRL --start
    sleep 1
    $OP_CNTRL --reset 
}
	

stop_oprofile() {
    check_oprofile
    mark_oprofile "Framework Close" 
    console_note "Ending the profile gathering phase"

    echo  "</table><p>" >> $OP_INDEX
    echo  "</body></html>" >> $OP_INDEX
    $OP_CNTRL --stop

	
    safe_exit
}

safe_exit() {

	mv $LOGFILE $TARG/.
	console_note "Exiting oprofile tool"

    exit 0
}


setup_oprofile () {
	cd $WRAP_HOME
	/usr/bin/stp_install_oprofile
	console_note "Finished oprofile setup"
	if [ ! -f $TARG ]; then
		echo "Making directory $TARG"
		mkdir $TARG
	fi
	if [ -f "/usr/local/sbin/modprobe" ]; then 
		/usr/local/sbin/modprobe -v oprofile
	else 
		modprobe -v oprofile
	fi
	ln -s /root/linux/vmlinux /boot/vmlinux
	# we won't check status for this
	console_note "Initalizing oprofile"
	$OP_CNTRL --vmlinux=/root/linux/vmlinux
	$OP_CNTRL --init
}
