# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

=head1 package STP::DB::HostState

Sets up the HostState table in the database, provides accessor functions
for same

=cut

package STP::DB::HostState;

@ISA = qw( STP::DB::Database );

use STP::DB::Database;

use strict;
use warnings;

my %STATES;
my %STATES_REV;

# Possible states for Host_state_uid in the host table
$STATES{ avail }   = "STP Available";
$STATES{ maint }   = "Maintenance";
$STATES{ inst }    = "Installing OS";
$STATES{ comp }    = "Compiling";
$STATES{ runtest } = "Running Test";
$STATES{ upload }  = "Uploading";
$STATES{ reboot } = "Rebooting";

# Reverse lookup table
$STATES_REV{ "STP Available" }     = "avail";
$STATES_REV{ "Maintenance" }   = "maint";
$STATES_REV{ "Installing OS" } = "inst";
$STATES_REV{ "Compiling" }     = "comp";
$STATES_REV{ "Running Test" }  = "runtest";
$STATES_REV{ "Uploading" }     = "upload";
$STATES_REV{ "Rebooting" } 	= "reboot";

BEGIN { }

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " host_state ";

    $self->{ fields } = "hs.uid, hs.descriptor";

    return $self;
}

=head1 FUNCTION getHostStateByUid

Accessor function

=cut 

sub getHostStateByUid {
    my ( $self, $uid ) = @_;

    die "getHostStateByUid() MISSING UID" unless $uid;

    my $query = qq(
		   SELECT $self->{ fields }
		   FROM $self->{ tableName } hs
		   WHERE hs.uid = $uid
		   );

    $self->_select( $query );

    my $state = $self->getProperty( 0, "descriptor" );

    return $STATES_REV{ $state };
}

=head1 Function getHostStateByCode 

Another wierd function: if the code is numeric, return right away,
otherwise do a database lookup and return the HostState string

=cut

sub getHostStateByCode {
    my ( $self, $code ) = @_;

    if ( $STATES{ $code } =~ /^\d+$/ ) {
        return $STATES{ $code };
    }

    my $query;

    $query =
      "SELECT uid FROM $self->{ tableName } WHERE descriptor = "
      . "'$STATES{ $code }'";

    $self->{ resultSet } = $self->_select( $query );

    $STATES{ $code } = $self->getProperty( 0, "uid" );

    die "bad code: '$code' [$query] " unless $STATES{ $code };

    return $STATES{ $code };
}

=head1 FUNCTION createTable

Why is this stuff in our object?

=cut

# create the table (only used at setup time)
sub createTable {
    my ( $self ) = @_;

    my $query = qq(
		    CREATE TABLE $self->{ tableName }
		    ( uid INT, descriptor VARCHAR(20) )
		   );

    $self->_update( $query );
}

=head1 FUNCTION populateTable
 
appears to be a wierd hack to get a table built
Origonal contains a typo, here corrected

=cut 

# populate the table with the default host states
sub populateTable {
    my ( $self ) = @_;

    for ( keys %STATES ) {
        $self->writeHostState( $STATES{ $_ } );
    }

    # $self->writeHostState( "STP" );
    # $self->writeHostState( "Maintenance" );
    # $self->writeHostState( "Installing OS" );
    # $self->writeHostState( "Building Kernel" );
    # $self->writeHostState( "Running Test" );
}

# this is a departure from the normal writeBlah method in the rest of
# the tables.  that is because the hoststate table is meant to pretty
# much be read only by the stp, and no updates should ever be done.
# but, if we let the code do it's thing, the insert's fail, because
# we are providing a uid.

=head1 FUNCTION writeHostState

Very un-clear comment. We appear to need a separate insert function for
this table, only on table creation. Wierd

=cut 

sub writeHostState {
    my ( $self, $descriptor ) = @_;

    my $query;

    $query =
      "INSERT INTO $self->{ tableName } ( descriptor ) "
      . "VALUES ('$descriptor')";

    $self->_update( $query );

    # return $uid;  # does anything check the return here???
}

END { }

1;
