# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

=head1 Package STP::DB::StateTimer

  inherits from DB::Database
  Uses STP::Util but we don't know why - 
  possibly dead code

=cut 

package STP::DB::StateTimer;

@ISA = qw( STP::DB::Database );

use strict;
use STP::DB::Database;
use POSIX qw(strftime);
use STP::Util::Config;

=head1 FUNCTION new

 defines a state timer object which is a row in the database
 state timer has host_uid host_state_uid, last update, retry count

=cut 

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " state_timer ";

    $self->{ fields } =
      "st.host_uid, st.host_state_uid, st.last_updated, " . "st.retry_count";

    return $self;
}

=head1 FUNCTION createTable

 only used on install

=cut

sub createTable {
    my ( $self ) = @_;

    my $query = qq( CREATE TABLE $self->{ tableName }
		    ( host_uid INT, host_state_uid INT, last_updated DATETIME, retry_count INT )
		    );

    $self->_update( $query );
}

# if we're setting a state timer, we want to delete anything previous
# with prejudice, and just insert.

=head1 FUNCTION setStateTimer

deletes all timers associated with the specified host
set a new timer at localtime
no return

=cut

sub setStateTimer {
    my ( $self, $host_uid, $host_state_uid, $retry_count ) = @_;

    my $query =
      qq( DELETE FROM $self->{ tableName } WHERE host_uid = $host_uid );

    $self->_update( $query );

    my $created_date = strftime( "%Y-%m-%d %H:%M:%S", localtime );

    $query =
qq( INSERT INTO $self->{ tableName } (host_uid, host_state_uid, last_updated, retry_count) VALUES ( $host_uid, $host_state_uid, '$created_date', $retry_count ) );

    $self->_update( $query );
}

=head1 FUNCTION getAllStateTimers

Accessor funtion
returns result set will all timers

=cut

sub getAllStateTimers {
    my ( $self ) = @_;

    my $query = qq( SELECT $self->{ fields }
		    FROM $self->{ tableName } st
		    ORDER BY st.last_updated );

    $self->{ resultSet } = $self->_select( $query );
}

1;
