# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::DB::Test;

@ISA = qw( STP::DB::Database );

use strict;
use warnings;
use STP::XML::Test;

BEGIN { }

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " test ";

    $self->{ fields } = "t.uid, t.descriptor, t.description, t.lilo_default";

    return $self;
}

# create the table (only used at setup time)
sub createTable {
    my ( $self ) = @_;

    my $query = qq(
		    CREATE TABLE $self->{ tableName }
		    ( uid INT, descriptor VARCHAR(255), description TEXT, 
		      lilo_default VARCHAR(255)
		    )
		   );

    $self->_update( $query );

    $query = qq( CREATE TABLE image
		 ( uid INT, descriptor VARCHAR(255), imagename VARCHAR(50) ) );
    $self->_update( $query );
}

sub getTestByUid {
    my ( $self, $uid ) = @_;

    my $query = qq(
		   SELECT $self->{ fields } 
		   FROM $self->{ tableName } t
		   WHERE t.uid = $uid
		   );

    $self->{ resultSet } = $self->_select( $query );
}

sub getTestByDescriptor {
    my ( $self, $descriptor ) = @_;

    my $query = qq(
		   SELECT $self->{ fields } 
		   FROM $self->{ tableName } t
		   WHERE t.descriptor = '$descriptor'
		   );

    $self->{ resultSet } = $self->_select( $query );
}

sub getAllTests {
    my ( $self ) = @_;

    # get all of the uid's
    my $query = qq(
		    SELECT uid
		    FROM $self->{ tableName }
		    ORDER BY descriptor
		   );

    $self->{ resultSet } = $self->_select( $query );

    return $self->arrayOfResults();
}

sub arrayOfResults() {
    my ( $self ) = @_;

    my $count = $self->getNumRows();

    # if there are no results, return an undef
    if ( $count == 0 ) {
        return undef;
    }

    my @data = ();

    for ( my $i = 0; $i < $count; $i++ ) {
        my $uid = $self->getProperty( $i, "uid" );

        # pass our config along
        my $t = new STP::DB::Test( $self->{ config } );

        $t->getTestByUid( $uid );
        $data[ $i ] = $t;
    }

    return @data;
}

sub writeTest {
    my ( $self, $uid, $descriptor, $description, $lilo_default ) = @_;

    my $query;

    # do we insert or update?
    if ( $uid == 0 ) {

        # we're inserting, so get a new uid
        $query = qq(
		    INSERT INTO $self->{ tableName }
		    ( descriptor, description, lilo_default )
		    VALUES
		    ( '$descriptor', '$description', '$lilo_default'
		    )
		   );
    }

    # update
    else {
        $query = qq(
		    UPDATE $self->{ tableName }
		    SET descriptor = '$descriptor', description = '$description',
		    lilo_default = '$lilo_default'
		    WHERE uid = $uid
		   );
    }

    $self->_update( $query );

    return $uid;
}

sub toXML {
    my ( $self, $number ) = @_;

    my $xml = new STP::XML::Test();

    $xml->setElementValue( "uid", $self->getProperty( $number, "uid" ) );
    $xml->setElementValue( "descriptor",
                           $self->getProperty( $number, "descriptor" ) );
    $xml->setElementValue( "description",
                           $self->getProperty( $number, "description" ) );
    $xml->setElementValue( "lilo_default",
                           $self->getProperty( $number, "lilo_default" ) );

    return $xml;
}

END { }

1;
