# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::DB::TestHostType;

@ISA = qw( STP::DB::Database );

use strict;
use warnings;

BEGIN { }

sub new {
    my $self = {};
    bless $self;

    shift;

    ( $self->{ config } ) = @_;

    $self::dbh     = "";
    $self::db_lock = 0;

    $self->{ tableName } = " test_host_type ";

    $self->{ fields } = "tt.uid, tt.test_uid, tt.host_type_uid";

    return $self;
}

# create the table (only used at setup time)
sub createTable {
    my ( $self ) = @_;

    my $query = qq(
		    CREATE TABLE $self->{ tableName }
		    ( uid INT, test_uid INT, host_type_uid INT
		    )
		   );

    $self->_update( $query );
}

sub getTestHostTypeByUid {
    my ( $self, $uid ) = @_;

    my $query = qq(
		   SELECT $self->{ fields } 
		   FROM $self->{ tableName } t
		   WHERE t.uid = $uid
		   );

    $self->{ resultSet } = $self->_select( $query );
}

sub getAllHostTypesForTest {
    my ( $self ) = @_;

    # get all of the uid's
    my $query = qq(
		    SELECT uid
		    FROM $self->{ tableName }
		    ORDER BY descriptor
		   );

    $self->{ resultSet } = $self->_select( $query );

    return $self->arrayOfResults();
}

sub arrayOfResults() {
    my ( $self ) = @_;

    my $count = $self->getNumRows();

    # if there are no results, return an undef
    if ( $count == 0 ) {
        return undef;
    }

    my @data = ();

    for ( my $i = 0; $i < $count; $i++ ) {
        my $uid = $self->getProperty( $i, "uid" );

        # pass our config along
        my $t = new STP::DB::Test( $self->{ config } );

        $t->getTestHostTypeByUid( $uid );
        $data[ $i ] = $t;
    }

    return @data;
}

sub writeTestHostType {
    my ( $self, $uid, $test_uid, $host_type_uid ) = @_;

    my $query;

    # do we insert or update?
    if ( $uid == 0 ) {

        # we're inserting, so get a new uid
        $query = qq(
		    INSERT INTO $self->{ tableName }
		    ( test_uid, host_type_uid )
		    VALUES
		    ( $test_uid, $host_type_uid )
		   );
    }

    # update
    else {
        $query = qq(
		    UPDATE $self->{ tableName }
		    SET test_uid = $test_uid, host_type_uid = $host_type_uid
		    WHERE uid = $uid
		   );
    }

    $self->_update( $query );

    return $uid;
}

END { }

1;
