# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

no strict 'refs';

package STP::Validation::Config;

# packages that contain our test routines must be included here
use STP::Validation::Basic;
use STP::Validation::Test;

# add test validations into the new subroutine
sub new {
    my $self = {};
    bless $self;

    # here's where we put in our definitions
    # test validation

    # Host.pm
    my @get_hosttypes = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "get_hosttypes" } = \@get_hosttypes;

    my @get_hostinfo = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "get_hostinfo" } = \@get_hostinfo;

    my @get_hosts_for_test = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "get_hosts_for_test" } = \@get_hosts_for_test;

    my @get_tests = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "get_tests" } = \@get_tests;

    my @get_hosttypes_for_test =
      ( \&STP::Validation::Basic::argumentCountTest );

    # TestRequest.pm
    $self->{ "get_hosttypes_for_test" } = \@get_hosttypes_for_test;

    my @get_test_request_for_host =
      ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "get_test_request_for_host" } = \@get_test_request_for_host;

    my @get_host_state = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "get_host_state" } = \@get_host_state;

    my @set_host_state = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "set_host_state" } = \@set_host_state;

    my @set_test_request_status =
      ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "set_test_request_status" } = \@set_test_request_status;

    my @upload_test_results = ( \&STP::Validation::Basic::argumentCountTest );

    $self->{ "upload_test_results" } = \@upload_test_results;

    return $self;
}

sub getTests {
    my ( $self, $testName ) = @_;

    return $self->{ $testName };
}

1;
