# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package STP::XML::HostInfo;
@ISA = qw( STP::XML::QDXml );

use strict;
use warnings;
use STP::XML::QDXml;
use STP::XML::QDXmlElement;

# config:
#
# <hostinfo>
#    <description></description>
#    <hostname></hostname>
#    <cpu></cpu>
#    <ram></ram>
#    <eth100></eth100>
#    <eth1000></eth1000>
#    <numdisks></numdisks>
#    <disk1>
#       <device></device>
#       <mountpoint></mountpoint>
#       <size></size>
#       <type></type>
#       <raid></raid>
#       <spindles></spindles>
#    </disk1>
#    <disk2>...</disk2>
# </hostinfo>

# most of this new is a copy of QDXml -- i hate perl 5's "object" model
sub new {

    # if we get a config file name passed, use it
    shift;
    my ( $configfile ) = @_;

    # instanciate the data, fill in the values
    my $self = {};
    bless $self;

    # create an array to hold the data
    $self->{ data } = [];

    # set our element name
    $self->{ elementName } = "hostinfo";

    # set up our config
    $self->addElement( "hostname", "" );
    $self->addElement( "cpu",      "" );
    $self->addElement( "ram",      "" );
    $self->addElement( "eth100",   "" );
    $self->addElement( "eth1000",  "" );
    $self->addElement( "numdisks", "0" );

    return $self;
}

sub addDiskInfo {
    my ( $self, $device, $mountpoint, $type, $size, $raid, $spindles ) = @_;

    # figure out what our count is, increment it
    my $count = $self->getElementValue( "numdisks" );

    $count++;

    # instanciate the disk section
    my $sub = new STP::XML::QDXml( "disk" . $count );

    # add the new elements
    $sub->addElement( "device",     $device );
    $sub->addElement( "mountpoint", $mountpoint );
    $sub->addElement( "type",       $type );
    $sub->addElement( "size",       $size );
    $sub->addElement( "raid",       $raid );
    $sub->addElement( "spindles",   $spindles );

    # add it in
    $self->addElement( $sub );

    # increment the count
    $self->setElementValue( "numdisks", $count );
}

sub parseXMLData {
    my ( $self, $xmlData ) = @_;

    # parse first, get the number of disks
    $self->SUPER::parseXMLData( $xmlData );

    # get the disk count
    my $count = $self->getElementValue( "numdisks" );

    # fix up the count
    $self->setElementValue( "numdisks", 0 );

    # create the disk entries
    for ( my $i = 0; $i < $count; $i++ ) {
        $self->addDiskInfo( "", "", "", "", "", "" );
    }

    # reparse to fill in the data
    $self->SUPER::parseXMLData( $xmlData );
}

1;
