# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Filename: QDXmlElement.pm
# Author: Jerry Sievert (jerry@osdl.org)
# Date: Feb 18, 2002
#
# Quick/Dirty XML Element
#
# These are the elements that make up and XML document.  There
# can be 0 or more XML Elements inside of an XML document.
#
# It's defined this way because it I wanted to make sure that
# the API matched the existing QDXml code package written in
# Java.  Having the same API under multiple languages is a
# major timesaver for me.

package STP::XML::QDXmlElement;

@ISA = qw( STP::XML::QDObject );

#use strict;
use warnings;
use STP::XML::QDObject;

BEGIN { }

# instanciate a new QDXmlElement
# possible arguments are as follows:
#
#   elementName
#   elementValue
sub new {

    # instanciate the data, fill in values
    my $self = {};
    bless $self;

    shift;

    my ( $elementName, $elementValue ) = @_;

    if ( defined( $elementName ) ) {
        $self->{ elementName } = $elementName;
    } else {

        # should be defined, even if empty
        $self->{ elementName } = "";
    }

    if ( defined( $elementValue ) ) {
        $self->{ elementValue } = $elementValue;
    } else {

        # also should be defined, even if empty
        $self->{ elementValue } = "";
    }

    return $self;
}

sub getDataType {
    my ( $self ) = @_;
    return 2;
}

# returns the name of the element being referenced, or if the case
# of a full document, the object
sub getElementName {
    my ( $self ) = @_;
    return $self->{ elementName };
}

sub setElementName {
    my ( $self, $elementName ) = @_;
    $self->{ elementName } = $elementName;
}

sub getElementValue {
    my ( $self ) = @_;
    return ${ $self->{ elementValue } };
}

sub setElementValue {
    my ( $self, $elementValue ) = @_;
    $self->{ elementValue } = \$elementValue;
}

sub toString {
    my ( $self, $depth ) = @_;

    # if we don't have an elementName, we just return nothing
    if ( $self->{ elementName } eq "" ) {
        return "";
    }

    my $data = "";

    # set our depth
    if ( defined( $depth ) && $depth > 0 ) {
        $data .= " " x $depth;
    }

    # add the elementName
    $data .= "<" . $self->{ elementName } . ">";

    # add the elementValue
    if ( defined( ${ $self->{ elementValue } } ) ) {
        $data .= ${ $self->{ elementValue } };
    }

    # add the closing elementName
    $data .= "</" . $self->{ elementName } . ">\n";

    return $data;
}

sub parseXMLData {
    my ( $self, $xmlData ) = @_;

    # if we have no elementName, simply return, doing nothing
    if ( $self->{ elementName } eq "" ) {
        return;
    }

    my $data =
      &STP::XML::QDXml::getBareElement( $self->{ elementName }, $xmlData );
    $self->{ elementValue } = \$data;

}

END { }

1;
