# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# an XML object for the transport schema

package STP::XML::Transport;
@ISA = qw( STP::XML::QDXml );

use strict;
use warnings;
use STP::XML::QDXml;
use STP::XML::QDXmlElement;

sub new {

    # instanciate the data, fill in values
    my $self = {};
    bless $self;

    # create an array to hold the data
    $self->{ data } = [];

    # set our element name
    $self->{ elementName } = "transport";

    # set up our fields
    $self->addElement( "version", "" );
    $self->addElement( "comment", "" );
    $self->addElement( "count",   "0" );

    return $self;
}

# create a new data-X subtree, with X set to the correct number
sub create_subtree {
    my $self = shift;

    my $number = shift;

    # create the new subtree
    my $data = new STP::XML::QDXml( "data-" . $number );

    # populate it
    $data->addElement( "comment", "" );
    $data->addElement( "type",    "" );
    $data->addElement( "data",    "" );

    $self->addElement( $data );
}

# autopopulate, using parseXMLData, getting the count for the number
# of subtrees to create
sub parseXMLData {
    my ( $self, $xmlData ) = @_;

    $self->SUPER::parseXMLData( $xmlData );

    my $count = $self->getElementValue( "count" );

    # do we have anything to populate?
    if ( $count > 0 ) {
        for ( my $i = 0; $i < $count; $i++ ) {
            $self->create_subtree( $i + 1 );
        }

        $self->SUPER::parseXMLData( $xmlData );
    }
}

sub getNumDatasets {
    my $self = shift;

    my $count = $self->getElementValue( "count" );
    return $count;
}

sub getDataset {
    my $self = shift;

    my $dataset = shift;

    if ( $dataset > $self->getNumDatasets() ) {
        return "";
    }

    return $self->getElementValue( "data-" . $dataset . ".data" );
}

sub addDataset {
    my ( $self, $type, $data ) = @_;

    my $count = $self->getElementValue( "count" );
    $count++;
    $self->setElementValue( "count", $count );

    $self->create_subtree( $count );

    $self->setElementValue( "data-" . $count . ".data", $data );
    $self->setElementValue( "data-" . $count . ".type", $type );
}

sub getDataType {
    my ( $self, $dataset ) = @_;

    my $count = $self->getNumDatasets();
    if ( $dataset > $count ) {
        return "";
    }

    return $self->getElementValue( "data-" . $count . ".type" );
}

1;
