#!/usr/bin/perl 
# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

require STP::Util::Config;
require STP::XML::Config;
require STP::DB::Database;
require STP::DB::HostType;
require STP::DB::Test;
require STP::DB::TestHostType;
require STP::DB::TestRequest;

use Date::Manip;

# get our config
my $config = STP::Util::Config::getConfig();

# where do we write the data?
my $datapath = $config->getElementValue( "report.datapath" );

# since we're going to be making a lot of random queries, get a db handle
my $db = new STP::DB::Database( $config );

# get a list of hosttypes
my $ht        = new STP::DB::HostType( $config );
my @hosttypes = $ht->getAllHostTypes();

outputDataSet1();

# output data for all hosttypes, priorities and statuses
# date,hosttype,status,priority,count,oldest,newest,average-age
sub outputDataSet1 {

    my $filename = $datapath . "/data1-hourly";

    # set up some variables we will want, like time
    my $curtime = time();

    # given our hosttypes, get the data for each priority and status
    my @statuses = ( "FAILED", "QUEUED", "Complete", "CANCELLED" );
    foreach my $ht ( @hosttypes ) {
        my $hosttype = $ht->getProperty( 0, "uid" );
        my $htdesc   = $ht->getProperty( 0, "descriptor" );
        foreach my $status ( @statuses ) {
            for ( my $priority = 1; $priority <= 3; $priority++ ) {
                my $res = getTestRequestsByPriorityHostTypeAndStatus(
                                                        $priority, $hosttype,
                                                        $status
                );
                my $count = $res->getNumRows();

                my @data = ();
                push ( @data, scalar localtime( $curtime ) );
                push ( @data, $htdesc );
                push ( @data, $status );
                push ( @data, $priority );
                push ( @data, $count );

                # if there are any there, put the oldest, newest, and ave in
                if ( $count != 0 ) {
                    my $cd = $res->getProperty( 0, "created_date" );
                    my $date = ParseDate( $cd );
                    push ( @data, scalar localtime( UnixDate( $date, "%s" ) ) );
                    $cd = $res->getProperty( $count - 1, "created_date" );
                    $date = ParseDate( $cd );
                    push ( @data, scalar localtime( UnixDate( $date, "%s" ) ) );

                    # get the averages
                    my @dp = ();
                    for ( my $i = 0; $i < $count; $i++ ) {
                        $cd = $res->getProperty( $i, "created_date" );
                        $date = ParseDate( $cd );
                        push ( @dp, UnixDate( $date, "%s" ) );
                    }
                    push ( @data, scalar localtime( average( @dp ) ) );
                } else {
                    push ( @data, "" );
                    push ( @data, "" );
                    push ( @data, "" );
                }

                output_data( $filename, @data );
            }
        }
    }
}

# output data for all hosttypes, priorities and statuses
# date,hosttype,status,priority,count,oldest,newest,average-age
# all within the last hour (we assume this is called every hour)
sub outputDataSet2 {

    my $filename = $datapath . "/data2-hourly";

    # set up some variables we will want, like time
    my $curtime = time();

    # given our hosttypes, get the data for each priority and status
    my @statuses = ( "FAILED", "QUEUED", "Complete", "CANCELLED" );
    foreach my $ht ( @hosttypes ) {
        my $hosttype = $ht->getProperty( 0, "uid" );
        my $htdesc   = $ht->getProperty( 0, "descriptor" );
        foreach my $status ( @statuses ) {
            for ( my $priority = 1; $priority <= 3; $priority++ ) {
                my $res =
                  getTestRequestsByPriorityHostTypeAndStatusWithinTimeframe(
                                                           $priority, $hosttype,
                                                           $status,   $timeframe
                );
                my $count = $res->getNumRows();

                my @data = ();
                push ( @data, scalar localtime( $curtime ) );
                push ( @data, $htdesc );
                push ( @data, $status );
                push ( @data, $priority );
                push ( @data, $count );

                # if there are any there, put the oldest, newest, and ave in
                if ( $count != 0 ) {
                    my $cd = $res->getProperty( 0, "created_date" );
                    my $date = ParseDate( $cd );
                    push ( @data, scalar localtime( UnixDate( $date, "%s" ) ) );
                    $cd = $res->getProperty( $count - 1, "created_date" );
                    $date = ParseDate( $cd );
                    push ( @data, scalar localtime( UnixDate( $date, "%s" ) ) );

                    # get the averages
                    my @dp = ();
                    for ( my $i = 0; $i < $count; $i++ ) {
                        $cd = $res->getProperty( $i, "created_date" );
                        $date = ParseDate( $cd );
                        push ( @dp, UnixDate( $date, "%s" ) );
                    }
                    push ( @data, scalar localtime( average( @dp ) ) );
                } else {
                    push ( @data, "" );
                    push ( @data, "" );
                    push ( @data, "" );
                }

                output_data( $filename, @data );
            }
        }
    }
}

# find test requests in the queue by priority, hosttype and status
sub getTestRequestsByPriorityHostTypeAndStatusWithinTimeframe {
    my ( $priority, $hosttype, $status, $time ) = @_;

    my $query =
qq( SELECT * FROM stp2_test_request WHERE status = '$status' AND test_priority = $priority AND host_type_uid = $hosttype AND created_date >= '$time' ORDER BY created_date);

    # do the query, return the resultset
    return $db->_select( $query );
}

# find test requests in the queue by priority, hosttype and status, created
# in the last hour
sub getTestRequestsByPriorityHostTypeAndStatus {
    my ( $priority, $hosttype, $status ) = @_;

    my $query =
qq( SELECT * FROM stp2_test_request WHERE status = '$status' AND test_priority = $priority AND host_type_uid = $hosttype ORDER BY created_date);

    # do the query, return the resultset
    return $db->_select( $query );
}

sub average {
    my $count = 0;
    my $data  = 0;
    foreach my $d ( @_ ) {
        $data += $d;
        $count++;
    }

    if ( $count == 0 ) {
        return 0;
    }

    return int( $data / $count );
}

sub output_data {
    my ( $filename, @data ) = @_;

    my $res = open( F, ">>$filename" );
    if ( !$res ) {
        print STDERR "unable to open file $filename for writing.\n";
        return;
    }

    my $dataset = join ( ",", @data );

    print F $dataset . "\n";

    close( F );
}
