#!/usr/bin/perl -w -I/usr/lib/perl5/site_perl/PLM
#

# This is a script to create a database insert for a list of auto-tests.
# We will use XML::Simple for this

use strict;
use BASE::Log;
use XML::Simple;
use Data::Dumper;
use Getopt::Long;
use PLM;

my $log  = getLog( "mass-request" );    # Global log object
my $user = 3125;                        # User to request tests as
my $plm;                                # PLM Tag to request against
my $req_list;
my $prio = 4;                               # Priority to add tests as
my $cfg  = getConfig();                     # Config handle
my $lilo = "profile=2 elevator=deadline";

GetOptions( "file=s" => \$req_list,
            "plm=i"  => \$plm );

my $dbh = new DB::Handle(
                          {
                            dsn  => $cfg->get( "GW:1:driver:dsn" ),
                            user => $cfg->get( "GW:1:driver:user" ),
                            pass => $cfg->get( "GW:1:driver:pass" )
                          }
);

$req_list = "/etc/stp_mass_request.xml" unless ( $req_list );

_panic( "bad syntax for mass-request script" ) unless ( $plm );
_panic( "generated invalid DBH" )              unless ( $dbh );

my $dbHandle = $dbh->connect();

my $ref = XMLin( $req_list );
my $dbref;
my $refkey;
my $sql;

# print Dumper( $ref );

foreach $refkey ( keys %{ $ref } ) {
    print "System $refkey\n";
    my $tst;
    my $host;

    # XML rules mean we can't name the XML == the database UID,
    # So we have to map XML -> host_type manually

SWITCH: {
    if ( $refkey eq "cpu1" ) 	 { $host = 1; last SWITCH; }
    if ( $refkey eq "stp2-000" ) { $host = 77; last SWITCH; }
    if ( $refkey eq "stp2-001" ) { $host = 78; last SWITCH; }
    if ( $refkey eq "stp2-002" ) { $host = 79; last SWITCH; }
    if ( $refkey eq "stp2-003" ) { $host = 80; last SWITCH; }
    if ( $refkey eq "stp4-000" ) { $host = 81; last SWITCH; }
    if ( $refkey eq "stp4-001" ) { $host = 82; last SWITCH; }
    if ( $refkey eq "stp4-002" ) { $host = 83; last SWITCH; }
    if ( $refkey eq "stp4-003" ) { $host = 84; last SWITCH; }
    if ( $refkey eq "stp8-000" ) { $host = 85; last SWITCH; }
    if ( $refkey eq "stp8-001" ) { $host = 86; last SWITCH; }
    if ( $refkey eq "stp8-002" ) { $host = 87; last SWITCH; }
    if ( $refkey eq "stp8-003" ) { $host = 88; last SWITCH; }
} 
    foreach $tst ( keys %{ $ref->{ $refkey } } ) {
        my $test     = $ref->{ $refkey }{ $tst }{ 'testID' };
        my $distroID = $ref->{ $refkey }{ $tst }{ 'distro' };

        if ( $ref->{ $refkey }{ $tst }{ 'lilo' } ) {
            $lilo = $ref->{ $refkey }{ $tst }{ 'lilo' };
        }
        my $opt = "";
        if ( $ref->{ $refkey }{ $tst }{ options } ) {

            my $lref = ref( $ref->{ $refkey }{ $tst }{ 'options' } );

            if ( $lref =~ /ARRAY/ ) {

                foreach $opt ( @{ $ref->{ $refkey }{ $tst }{ 'options' } } ) {
                    my $request =
                      $dbHandle->prepare( "INSERT INTO test_request "
                        . "(rsf, created_by, created_date, status, distro_tag_uid, "
                        . " patch_tag, test_uid, auto_publish, test_priority, host_uid,  "
                        . " host_type_uid, last_updated_by, last_updated_date, project_uid, lilo, script_param ) "
                        . "VALUES (1, $user, NOW(), 'Queued', $distroID, '$plm',"
                        . " $test, 1, $prio, 0, $host, $user, NOW(), 1074, \'$lilo\','$opt')"
                      );
                    $request->execute();
                    $request->finish();
                    print "Host type $host ";
                    print "Test UID $test ";
                    print "Test name: $tst  ";
                    print "Test ID: $test  ";
                    print "Script opts: $opt  ";
                    print "lilo: $lilo\n";
                }
            } else {
                my $request =
                  $dbHandle->prepare( "INSERT INTO test_request "
                    . "(rsf, created_by, created_date, status, distro_tag_uid, "
                    . " patch_tag, test_uid, auto_publish, test_priority, host_uid,  "
                    . " host_type_uid, last_updated_by, last_updated_date, project_uid, lilo, script_param ) "
                    . "VALUES (1, $user, NOW(), 'Queued', $distroID, '$plm',"
                    . " $test, 1, $prio, 0, $host, $user, NOW(), 1074, \'$lilo\','$ref->{ $refkey }{ $tst }{ options }')"
                  );
                $request->execute();
                $request->finish();
                print "Host type $host ";
                print "Test UID $test ";
                print "Test name: $tst  ";
                print "Test ID: $test  ";
                print "Script opts: $ref->{ $refkey }{ $tst }{ 'options' }  ";
                print "lilo: $lilo\n";
            }

        } else {
            my $request =
              $dbHandle->prepare( "INSERT INTO test_request "
                . "(rsf, created_by, created_date, status, distro_tag_uid, "
                . " patch_tag, test_uid, auto_publish, test_priority, host_uid,  "
                . " host_type_uid, last_updated_by, last_updated_date, project_uid, lilo, script_param ) "
                . "VALUES (1, $user, NOW(), 'Queued', $distroID, '$plm',"
                . " $test, 1, $prio, 0, $host, $user, NOW(), 1074, \'$lilo\','$opt')"
              );
            $request->execute();
            $request->finish();

            print "Test name: $tst  ";
            print "Test ID: $test  ";
            print "Script opts: $opt  ";
            print "lilo: $lilo\n";
        }

    }
}

sub _panic {
    my $txt = shift;
    chomp $txt;
    $log->msg( 0, "PANIC: $txt" );
    print STDERR "PANIC: $txt\n";

    exit 1;
}
