#!/usr/bin/perl

#
#  This script attempts to cycle a host, if it fails it returns a non-zero value.
#

use strict;
use Expect;


use Getopt::Long;

my $config_file='/etc/stp/stp_power_soft.cfg';
my $command;
my $command_output;
my $feedback=0;
my $host;
my $host_match;
my $input;
my $login;
my $option;
my $option_match;
my $password;
my $timeout=60;
my $shell_prompt;
my $reboot_file='already_soft_rebooted';
# Expect catches our command too, so do not look for value of $reboot_file
my $reboot_text='XXX';


GetOptions( "option|o=s" => \$option,
	     "host|=s" => \$host );

if ( ! $option || ! $host ) {
    die "usage: power_soft.pl --option <command option> --host <host>\n";
}

# Set up

if (open( CFG, "<${config_file}" )) {
    while ( $input=<CFG> ){
        chomp $input;
        ($host_match, $login, $password, $shell_prompt, $option_match, $command, $command_output) = split /\,/, $input;
        if ( $host =~ m/$host_match/ && $option =~ m/$option_match/ ) {
            $feedback = 1;
            last;
        }
    }
} else {
    print "Could not open file: ${config_file}\n";
    exit -1;
}

if ( $feedback == 0 ) {
    print "No matches for $host\n";
    exit $feedback;
}

# Begin the actual Expect portion
# Connect to the host
my $exp = Expect->spawn( "ssh \-l ${login} ${host}" ) or die "Cannot spawn ssh: $!\n";

#  Login and shutdown 
my $patidx = $exp->expect($timeout, 
                [ qr/authenticity of host/, sub { my $self = shift; $self->send("yes\n"); exp_continue; }],
                [ qr/password:/i, sub { my $self = shift; $self->send("$password\n"); }]);

#
# This section is to check if we have attempted soft boot already
#
$exp->expect($timeout ,
                [ qr/$shell_prompt/i, sub { my $self = shift;$self->send("cat $reboot_file\n"); }] );
# This catches our command too, so do not look for value of $reboot_file
$exp->expect($timeout , 
                [qr/${reboot_text}|${shell_prompt}/] );
my $match_result=$exp->exp_match();
# print STDERR "match_result \"$match_result\" \n";

if (! $match_result ){
       #  This should not be null unless host unresponsive.
       die "No response on ${host}.\n";
} elsif ($match_result=~m/$reboot_text/) {
       # Soft boot was already attempted
       $exp->send( "exit\n" );
       $exp->soft_close();
       die "Soft boot of ${host} already attempted.\n";
} elsif ($match_result=~m/$shell_prompt/) {
       # No file present, make one and attempt a reboot
       $exp->send( "echo $reboot_text > $reboot_file\n" );
       my $patidx = $exp->expect($timeout,
                [ qr/$shell_prompt/i, sub { my $self = shift; $self->send("$command\n"); exp_continue; }] ,
                [ qr/$command_output/i] );
        # This will catch the last match error.
        my $feedback = $exp->error();

        # Logoff
        $exp->send( "exit\n" );
        # if no longer needed, do a soft_close to nicely shut down the command
        $exp->soft_close();

        exit $feedback;
} else {
        # This should not happen
        $exp->soft_close();
        die "Strange match:  $match_result on ${host}\n";
}

# We should never get here either
exit -1;
