#!/usr/bin/perl -w -I/usr/lib/perl5/site_perl/PLM
#

# This is a script to create a database insert for a list of auto-tests.
# We will use XML::Simple for this
# Cliffw OSDL 2004

#Copyright (C) 2004 Open Source Development Lab 
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#


use strict;
use XML::Simple;
use Data::Dumper;
use Getopt::Long;
use FileHandle;


my $user = 3125;                        # User to request tests as
my $outfile;                                # PLM Tag to request against
my $req_list;
my $prio = 4;                               # Priority to add tests as
my $lilo = "profile=2 elevator=deadline";

GetOptions( "file=s" => \$req_list,
            "outfile=s"  => \$outfile );


my $ref = XMLin( $req_list );
my $dbref;
my $refkey;
my $sql;

# print Dumper( $ref );

my $of = new FileHandle;
unless ( $of->open( "> $outfile" ) ) {
	die "Missing output file $!";
}

print $of "<html><title>Test Schedule report</title><body>\n";
print $of "<h2>Tests to be run for file: $req_list</h2>\n";

foreach $refkey ( sort (keys %{ $ref }) ) {
    print $of "<h3> System $refkey </h3><br>\n";
    print $of "<table border=2 cellspacing=2 cellpadding=2 caption='System $refkey'>\n";
    
    print $of "<tr><th> Test Name </th><th> Script Opts </th><th> Lilo Opts </th></tr>\n";
    my $tst;
    my $host;

    # XML rules mean we can't name the XML == the database UID,
    # So we have to map XML -> host_type manually

    foreach $tst ( keys %{ $ref->{ $refkey } } ) {
        my $test     = $ref->{ $refkey }{ $tst }{ 'testID' };
        my $distroID = $ref->{ $refkey }{ $tst }{ 'distro' };

        if ( $ref->{ $refkey }{ $tst }{ 'lilo' } ) {
            $lilo = $ref->{ $refkey }{ $tst }{ 'lilo' };
        }
        if ( $ref->{ $refkey }{ $tst }{ 'priority' } ) {
            $prio = $ref->{ $refkey }{ $tst }{ 'priority' };
        }
        my $opt = "";
        if ( $ref->{ $refkey }{ $tst }{ options } ) {

            my $lref = ref( $ref->{ $refkey }{ $tst }{ 'options' } );

            if ( $lref =~ /ARRAY/ ) {

                foreach $opt ( @{ $ref->{ $refkey }{ $tst }{ 'options' } } ) {
                    print $of "<tr valign=\"TOP\">";
	 	    print $of "<td> $tst </td><td> $opt </td><td>$lilo</td></tr>";		
                }
            } else {
                    print $of "<tr valign=\"TOP\">";
	 	    print $of "<td> $tst </td>\n";
		    print $of "<td> $ref->{ $refkey }{ $tst}{ 'options' } </td>\n";
	            print $of "<td>$lilo</td></tr>\n";
            }

        } else {
                    print $of "<tr valign=\"TOP\">";
	 	    print $of "<td> $tst </td><td> $opt </td><td>$lilo</td></tr>";		

        }

    }
	print $of "</table>\n";
}
print $of "</body></html>\n";

$of->close();

sub _panic {
    my $txt = shift;
    chomp $txt;
    print STDERR "PANIC: $txt\n";

    exit 1;
}
