#!/usr/bin/perl -w -I/usr/lib/perl5/site_perl/

exit(0) if ( -f "/var/lock/stp-stop" );

#
# Copyright (C) 2002 Open Source Development Lab 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# fixes from cliffw

use STP::XML::Config;
use STP::DB::TestRequest;
use STP::DB::Test;
use STP::DB::HostType;
use STP::DB::HostState;
use STP::DB::Host;
use STP::DB::StateTimer;
use STP::Util::Config;
use Mail::Mailer;
use Date::Manip;

use strict;

use constant DEBUG => 1;
#change to 0 to remove print output

my $config = STP::Util::Config::getConfig();

my $host_type  = new STP::DB::HostType( $config );
my $host_state = new STP::DB::HostState( $config );
my $test       = new STP::DB::TestRequest( $config );
my $host       = new STP::DB::Host( $config );
my $timer      = new STP::DB::StateTimer( $config );
my $user       = new STP::DB::User( $config );
my $test_info  = new STP::DB::Test( $config );

# first cancel tests
my $cancel_count = cancel_tests();

print "$cancel_count Tests Canceled\n" if DEBUG;

# check the state timers
check_state_timer();

# for each one, find out if there is a test waiting for each priority
# until we run out of hosts
my @host_type_list = $host_type->getAllHostTypes();

foreach my $type ( @host_type_list ) {
    my $type_uid = $type->getProperty( 0, "uid" );

    for ( my $priority = 1; $priority <= 4; $priority++ ) {
        $test->getOldestQueuedTestByPriority( $priority, $type_uid );

        # if we have a test, see if we have a machine to queue it on
        for ( my $i = 0; $i < $test->getNumRows(); $i++ ) {
            $host->getFreeHostsByHosttype( $type_uid );

            if ( $host->getNumRows() != 0 ) {
                my $new_test_uid = $test->getProperty( $i, "uid" );
                my $new_host_uid = $host->getProperty( 0,  "uid" );

                # queue the test, start the process
                queue_test_on_host( $new_test_uid, $new_host_uid );
            }

        }
    }
}

sub queue_test_on_host {
    my ( $test_uid, $host_uid ) = @_;
    my $rv=0;

    # TODO - remove this global data, pass into function
    $test->getTestRequestByUid( $test_uid );

    die "queue_test_on_host() FAILED to load test [$test_uid]" unless $test;

    print "instantiated testrequest\n";

    # update the test
    # TODO - update the request status _after 
    # the assign succeeds, instead of before

    $test->updateTestRequestStatus( "Installing Operating System" );
    print "updated test request status\n";
    $rv = $test->assignTestRequestToHost( $test_uid, $host_uid );
    if ($rv != 0) {
	print "Unable to assignTestRequestToHost.";
	print "Requeueing the test request\n";
        $test->updateTestRequestStatus( "Queued" );
	return;
    }
    print "assigned test request $test_uid to host $host_uid\n";

    # update the state of the host
    $host->getHostByUid( $host_uid );

    print "instantiated host\n";

    $host->setHostState( "inst" );

    # Find the command we are supposed to use to install the distro
    my $command = $test->getInstallCommand( $test_uid, $host_uid );
    chomp $command;

    if ( install_host( $host_uid, $command ) != 0 ) {
        print "ERROR: Unable to request OS install on host: $host_uid\n";
        
        print "Requeueing the test request\n";
        $test->getTestRequestByUid( $test_uid );
        $test->updateTestRequestStatus( "Queued" );

        print "Setting the host to state 'maint'\n";
        $host->getHostByUid( $host_uid );
        $host->setHostState( "maint" );
    }
}

sub install_host {
    my ( $host_uid, $command ) = @_;

    $host->getHostByUid( $host_uid );

    my $power    = $config->getElementValue( "utility.power" );
    my $hostname = $host->getProperty( 0, "descriptor" );

    # if there is no reimaging program, we email the admin to do it
    # we also email the admin when the command includes the text 'mail'
    if ( ( !$command ) || ( $command =~ "mail" ) ) {
        print "STP: INSTALL [$hostname] [$command]\n";
        mail_admin( "STP: INSTALL on [$hostname] command: [$command]", "[STP] Install Request Failed" );
        return 0;
    }

    # Time to do the actual install/reboot
    print "system [$command] [$hostname]\n";
    system "$command $hostname";
    if ( $? ) {
        print "OS install request returned an error for [ $hostname ]\n";
        mail_admin( "OS install request returned an error for [ $hostname ]\n"
                  . "NOTICE: The host has been put into Maintenance mode",
                    "[STP] PANIC: $hostname install error (Maintenance)" );
        return 1;
    }
    
    # Force the remote machine to reboot
    print "system [$power] [cycle] [$hostname]\n";
    system "$power cycle $hostname";
    if ( $? ) {
        print "Power cycle request returned an error for [ $hostname ]\n";
        mail_admin( "Power cycle request returned an error for [ $hostname ]\n"
                  . "NOTICE: The host has been put into Maintenance mode",
                    "[STP] PANIC: $hostname power cycle error (Maintenance)" );
        return 1;
    }

    return 0;
}

sub mail_admin {
    my ( $message,$subject )= @_;

    my $admin = $config->getElementValue( "email" );

    my $mailer = Mail::Mailer->new( "sendmail" );
    $mailer->open(
        {
            From    => "stp",
            To      => $admin,
            Subject => $subject || "STP Automated Notification",
        }
    );

    print $mailer $message;
    $mailer->close();
}

sub mail_user {
    my ( $subject, $message ) = @_;

    my $test_uid = $test->getProperty( 0, "uid" );
    unless ( $test_uid ) {
        print "mail_user() Unable to find test uid.\n";
        return;
    }

    my $created_by = $test->getProperty( 0, "created_by" );
    unless ( $created_by ) {
        print "mail_user() Unable to find created_by for test $test_uid.\n";
        return;
    }

    my $admin = $config->getElementValue( "email" );
    unless ( $admin ) {
        print "mail_user() Unable to find admin email address.\n";
        return;
    }

    $user->getUserByUid( $created_by );
    unless ( $user ) {
        print
          "mail_user() Unable to load user via getUserByUid( $created_by ).\n";
        return;
    }

    my $email = $user->getProperty( 0, "real_email" );
    $email = $user->getProperty( 0, "Real_email" ) unless ( $email );
    unless ( $email ) {
        print "mail_user() Unable to find user's email address.\n";
        print "  - test_uid   = $test_uid\n";
        print "  - created_by = $created_by\n";
        print "  - admin      = $admin\n";
        return;
    }

    my $mailer = Mail::Mailer->new( "sendmail" );

    print "Sending email to $email from $admin\n";

    $mailer->open(
        {
            From    => $admin,
            To      => $email,
            Subject => $subject || "STP Automated Message"
        }
    );

    print $mailer $message;

    $mailer->close();
}

sub cancel_tests {
    my $power = $config->getElementValue( "utility.power" );
    my $image = $config->getElementValue( "utility.image" );

    my $numRows = $test->getCancelledTests();
    if ( $numRows == 0 ) { return 0; }
	print "rows found $numRows\n" if DEBUG;
    my $count = 0; 

    for ( my $i = 0; $i < $numRows; $i++ ) {
        my $testuid = $test->getProperty( $i, "uid" );
	unless ( $testuid ) { 
		print "Bogus row! FIX GETNEXT ROW\n";
		next;
	}
        print "Cancelling Test: $testuid\n";

        my $id = $host->getHostForTestRequest( $testuid );
	print "Cancel test: id $id\n" if DEBUG;

        # error checking
        if ( $host->getNumRows() == 0 ) {
            print "cancelled test not assigned to host:\n";
            print "(test uid $testuid)\n";
        } else {

            # if there is no power program, we email the admin to do it
            if ( $power eq "" ) {
                mail_admin( "Power Down Host (test cancelled): "
                          . $host->getProperty( 0, "descriptor" ),
                            "Host power reset request" );
            } else {
                system $image, $host->getProperty( 0, "descriptor" );

                # Turning the power off kills some hardware
                # system $power, "off", $host->getProperty( 0, "descriptor" );
            }
            print "updating host\n";
            $host->getHostByUid( $id );
            $host->setHostState( "avail" );

            print "updating test request\n";
            $test->updateTestRequestStatus( "CANCELLED" );

            print "cancelled test: $testuid\n";
	    $count++;
        }
    }
return $count;
}

sub check_state_timer {
    my $MINUTE = 60;
    my $HOUR   = $MINUTE * 60;
    my $DAY    = $HOUR * 24;

    $timer->getAllStateTimers();
    my $count = $timer->getNumRows();
    print "Timers found: $count\n" if DEBUG;

    for ( my $i = 0; $i < $count; $i++ ) {
        my $host_uid       = $timer->getProperty( $i, "host_uid" );
        my $host_state_uid = $timer->getProperty( $i, "host_state_uid" );
        my $last_updated   = $timer->getProperty( $i, "last_updated" );
        my $retry          = $timer->getProperty( $i, "retry_count" );

	# if timer is invalid, REPORT AN ERROR
	#
        if ( $host_state_uid && $host_state && $host_uid ) {

        # get the current timer information
        my $cur_state = $host_state->getHostStateByUid( $host_state_uid );
	print "Current state $cur_state\n" if DEBUG;
        unless ( $cur_state ) {
		print "Problem with host_state_uid $host_state_uid\n";
		next;
	}

        # parse the date to epoch
        my $date      = ParseDate( $last_updated );
        my $epochtime = UnixDate( $date, "%s" );

        my $cur_time = time();

        # 1 hour for OS install 
        if ( $cur_state eq "inst" ) {
            if ( $cur_time > ( $epochtime + $HOUR ) ) {
                kick_machine( $host_uid, $cur_state, $retry );
            }
        }

        # 1 hour & 30 minutes for building kernel (includes reboot...)
        if ( $cur_state eq "comp" ) {
            if ( $cur_time > ( $epochtime + $HOUR + $MINUTE * 30 ) ) {
                kick_machine( $host_uid, $cur_state, $retry );
            }
        }
	# 30 minutes for rebooting
	if ( $cur_state eq "reboot" ) {
	    if ( $cur_time > ( $epochtime + $MINUTE * 30 ) ) {
                kick_machine( $host_uid, $cur_state, $retry );
            }
        }

			

        # 1 day and 30 minutes for tests
        # long running tests should have cleaned themselves up by this point
        if ( $cur_state eq "runtest" ) {
            if ( $cur_time > ( $epochtime + ( $HOUR * 13 ) + ( $MINUTE * 30 ) ) ) {
                kick_machine( $host_uid, $cur_state, $retry );
            }
        }

        # 1 hour for upload timer
        if ( $cur_state eq "upload" ) {
            if ( $cur_time > ( $epochtime + $HOUR ) ) {
                kick_machine( $host_uid, $cur_state, $retry );
            }
        }
    } else {
	print "Invalid timer found at count $count\n";
	print "One of the following should be bad\n";
	print "Host UID = $host_uid\n";
	print "Host State = $host_state_uid\n";
	print "Last Updated = $last_updated\n";
	print "Retry count = $retry\n";
	}
  } 
}

sub kick_machine {

    my $ltest       = new STP::DB::TestRequest( $config );
    my $ltest_info  = new STP::DB::Test( $config );
    my ( $host_uid, $cur_state, $retry ) = @_;

    # increment the retry
    $retry++;

    # find out which test request we are running
    $ltest->getTestRequestForHostUid( $host_uid );
    my $test_uid = $ltest->getProperty( 0, "uid" );
    $ltest_info->getTestByUid( $test_uid );
    my $test_name = $ltest_info->getProperty( 0, "descriptor" );
    

    $host->getHostByUid( $host_uid );
    
    # fill out test failure mail properly
    my $patch = $ltest->getProperty( 0, "patch_tag" );
    my $host_name = $host->getProperty( 0, "descriptor" );
    my $kernel_info = new STP::DB::PatchTag( $config, $patch );
    my $kernel_name = $kernel_info->getProperty( 0, "descriptor" );
    

    # Trigger failure for non-critical failure types
    if ( ( $retry >= 1 ) && ( $cur_state eq "comp" ) ) {
        mail_user( "Test ID $test_uid \n STP Host: $host_name \n",
	    "The STP was unable to complete the kernel compile "
            . "of PLM #$patch or the resulting kernel ( $kernel_name ) was unbootable \n\n"
	    . "The system was unreachable and unfortunately no further data"
 	    . " was recovered.\n"
            . "It possible this was an internal failure, in this case the "
	    . "STP team should be working on the issue, and this mail may be "
	    . "unecessary \n\n"
            . "Please email the admin if you have any questions " 
            . "or require more information.\n" );

        $host->setHostState( "avail" );
        $ltest->updateTestRequestStatus( "Failed" );
	test_failed_report($test_uid, "Unknown failure while compiling the kernel.  [timeout]");
    }

    # Trigger for "runtest" failure type
    if ( ( $retry >= 1 ) && ( $cur_state eq "runtest" ) ) {
        mail_user( "Test ID $test_uid \n STP Host: $host_name \n",
		    "The $test_name test on $kernel_name failed to complete in time and has been marked "
                   . "as failed. (Typical timeout is 24 hours)  The reason the test failed to complete is "
                   . "unknown.\n\n"
                   . "Please email the admin if you have any questions." );

        $host->setHostState( "avail" );
        $ltest->updateTestRequestStatus( "Failed" );
	test_failed_report($test_uid, "Test failed to complete within time limit.");
    }

    # Trigget failure for critical failure types
    if ( $retry >= 4 ) {
        mail_admin( "Maximum Retry Reached: $host_name [$host_uid]\n"
                    . "host state = $cur_state\n"
                    . "test request ID = $test_uid\n"
                    . "\n*** Setting Maintanence State ***\n"
	            . "PLM Patch: $patch\n"
                    . "Kernel Name: $kernel_name\n"
                    . "Test being attempted: $test_name\n"
                    . "[STP $test_uid] PANIC: $host_name failure ($cur_state => Maintenance)" ,
		    "STP Failure (CRITICAL) - Test ID $test_uid" );

        $host->setHostState( "maint" );
        $ltest->updateTestRequestStatus( "Queued" );
        return;
    }

    #mail_admin( "Kicking: $host_name state=$cur_state retry=$retry\n",
    #            "[STP $test_uid] NOTICE: kicking host $host_name" );

    # set the new state timer
    my $host_state_uid = $host_state->getHostStateByCode( $cur_state );

    $timer->setStateTimer( $host_uid, $host_state_uid, $retry );

    # power cycle the host
    my $power = $config->getElementValue( "utility.power" );

    if ( ( $power eq "" ) || ( !-f $power ) ) {
        mail_admin( "Please Power-cycle Host: $host_name",
                    "[STP $test_uid] Please power-cycle $host_name" );
        return;
    }

    system $power, "cycle", $host_name;
}

sub test_failed_report {
    my $test_request_uid = shift || return;
    my $reason = shift || "unknown";

    my $datapath = $config->getElementValue( "test.resultpath" );
    my $tdir = "$datapath/$test_request_uid";

    $test->getTestRequestByUid( $test_request_uid );

    my $kernel = $test->getProperty( 0, "patch_tag" );
    
    $host->getHostByUid( $test->getProperty( 0, "host_uid" ) );
    my $hostname = $host->getProperty( 0, "descriptor" );

    my $test_case = new STP::DB::Test( $config );
    $test_case->getTestByUid( $test->getProperty( 0, "test_uid" ) );
    my $testname = $test_case->getProperty( 0, "descriptor" );
    
    open(R, ">$tdir/index.html") || return;

    print R "<html>\n<head>\n<title>STP Test Failed</title>\n</head>\n<body>\n";
    print R "<h2><center>Test Request $test_request_uid Failed</center></h2>";
    print R "This is a server-side failure (the host running the test because ";
    print R "unresponsive) so we are not able to provide the regular debug information.<p>\n";
    print R "The server gave the following reason for the test failure:<p>\n";
    print R "<pre>$reason</pre><p>\n";
    print R "<hr><p>\n";
    print R "Specifics: test $testname running kernel ID $kernel on host $hostname.\n"; 
    print R "</body>\n</html>";

    close (R);
}
